@@ -49,15 +49,16 @@
 
 package com.openexchange.filemanagement.distributed.servlet;
 
-import java.io.BufferedInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.util.Optional;
 import javax.servlet.ServletException;
 import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import com.openexchange.exception.OXException;
+import com.openexchange.filemanagement.DistributedFileUtils;
 import com.openexchange.filemanagement.ManagedFile;
 import com.openexchange.filemanagement.ManagedFileManagement;
 import com.openexchange.java.Streams;
@@ -70,23 +71,69 @@ import com.openexchange.server.ServiceLookup;
  */
 public class DistributedFileServlet extends HttpServlet {
 
-    private static final long serialVersionUID = 1L;
+    private static final long serialVersionUID = -3293278521646131568L;
 
-    private final ManagedFileManagement fileManagement;
+    /** Simple class to delay initialization until needed */
+    private static class LoggerHolder {
+        static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(DistributedFileServlet.class);
+    }
 
-    public static final String PATH = "distributedFiles";
+    private final ServiceLookup services;
 
+    /**
+     * Initializes a new {@link DistributedFileServlet}.
+     *
+     * @param services The service look-up
+     */
     public DistributedFileServlet(ServiceLookup services) {
-        fileManagement = services.getService(ManagedFileManagement.class);
-//        port = services.getService(ConfigurationService.class).getIntProperty("com.openexchange.filemanagement.distributed.port", 2003);
+        super();
+        this.services = services;
+    }
+
+    private Optional<ManagedFileManagement> getFileManagement(HttpServletResponse resp) throws IOException {
+        try {
+            return Optional.of(services.getServiceSafe(ManagedFileManagement.class));
+        } catch (Exception e) {
+            resp.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, "Missing service");
+            return Optional.empty();
+        }
+    }
+
+    private Optional<String> extractId(String uri, HttpServletResponse resp) throws IOException {
+        int pos = uri.lastIndexOf('/');
+        if (pos < 0) {
+            return Optional.of(uri);
+        }
+
+        String extractedId = uri.substring(pos + 1);
+        return decodeId(extractedId, resp);
+    }
+
+    private Optional<String> decodeId(String encodedId, HttpServletResponse resp) throws IOException {
+        try {
+            return Optional.of(services.getServiceSafe(DistributedFileUtils.class).decodeId(encodedId));
+        } catch (Exception e) {
+            resp.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, "Missing service");
+            return Optional.empty();
+        }
     }
 
     @Override
     protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
-        String uri = req.getRequestURI();
-        String id = crop(uri);
+        Optional<String> optionalId = extractId(req.getRequestURI(), resp);
+        if (!optionalId.isPresent()) {
+            return;
+        }
+        String id = optionalId.get();
 
+        Optional<ManagedFileManagement> optionalFileManagement = getFileManagement(resp);
+        if (!optionalFileManagement.isPresent()) {
+            return;
+        }
+
+        ManagedFileManagement fileManagement = optionalFileManagement.get();
         if (!fileManagement.containsLocal(id)) {
+            resp.sendError(HttpServletResponse.SC_NOT_FOUND, "No such file");
             return;
         }
 
@@ -94,33 +141,38 @@ public class DistributedFileServlet extends HttpServlet {
         OutputStream outStream = null;
         try {
             ManagedFile file = fileManagement.getByID(id);
-            inStream = new BufferedInputStream(file.getInputStream(), 65536);
+            inStream = file.getInputStream();
             resp.setContentType(file.getContentType());
 
             outStream = resp.getOutputStream();
-            int bytesRead = 0;
-
-            while ((bytesRead = inStream.read()) != -1) {
-                outStream.write(bytesRead);
+            int buflen = 65536;
+            byte[] buf = new byte[buflen];
+            for (int read; (read = inStream.read(buf, 0, buflen)) > 0;) {
+                outStream.write(buf, 0, read);
             }
+            outStream.flush();
         } catch (OXException e) {
-            org.slf4j.LoggerFactory.getLogger(DistributedFileServlet.class).error("", e);
+            LoggerHolder.LOG.error("", e);
         } finally {
             Streams.close(inStream);
             Streams.close(outStream);
         }
     }
 
-    private String crop(String uri) {
-        String[] split = uri.split("/");
-        return split[split.length-1];
-    }
-
     @Override
     protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
-        String uri = req.getRequestURI();
-        String id = crop(uri);
+        Optional<String> optionalId = extractId(req.getRequestURI(), resp);
+        if (!optionalId.isPresent()) {
+            return;
+        }
+        String id = optionalId.get();
 
+        Optional<ManagedFileManagement> optionalFileManagement = getFileManagement(resp);
+        if (!optionalFileManagement.isPresent()) {
+            return;
+        }
+
+        ManagedFileManagement fileManagement = optionalFileManagement.get();
         if (!fileManagement.containsLocal(id)) {
             return;
         }
@@ -128,15 +180,24 @@ public class DistributedFileServlet extends HttpServlet {
         try {
             fileManagement.getByID(id);
         } catch (OXException e) {
-            org.slf4j.LoggerFactory.getLogger(DistributedFileServlet.class).error("", e);
+            LoggerHolder.LOG.error("", e);
         }
     }
 
     @Override
     protected void doDelete(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
-        String uri = req.getRequestURI();
-        String id = crop(uri);
+        Optional<String> optionalId = extractId(req.getRequestURI(), resp);
+        if (!optionalId.isPresent()) {
+            return;
+        }
+        String id = optionalId.get();
 
+        Optional<ManagedFileManagement> optionalFileManagement = getFileManagement(resp);
+        if (!optionalFileManagement.isPresent()) {
+            return;
+        }
+
+        ManagedFileManagement fileManagement = optionalFileManagement.get();
         if (!fileManagement.containsLocal(id)) {
             return;
         }
@@ -144,7 +205,7 @@ public class DistributedFileServlet extends HttpServlet {
         try {
             fileManagement.removeByID(id);
         } catch (OXException e) {
-            org.slf4j.LoggerFactory.getLogger(DistributedFileServlet.class).error("", e);
+            LoggerHolder.LOG.error("", e);
         }
     }
 }
