@@ -67,7 +67,9 @@ import com.hazelcast.core.HazelcastInstanceNotActiveException;
 import com.hazelcast.core.IMap;
 import com.openexchange.exception.OXException;
 import com.openexchange.filemanagement.DistributedFileManagement;
+import com.openexchange.filemanagement.DistributedFileUtils;
 import com.openexchange.filemanagement.ManagedFileExceptionErrorMessage;
+import com.openexchange.java.Streams;
 import com.openexchange.server.ServiceExceptionCode;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.threadpool.ThreadPoolService;
@@ -85,8 +87,6 @@ public class DistributedFileManagementImpl implements DistributedFileManagement
 
     private static AtomicReference<HazelcastInstance> REFERENCE = new AtomicReference<HazelcastInstance>();
 
-    private static final String PATH = "distributedFiles";
-
     public static void setHazelcastInstance(HazelcastInstance hazelcastInstance) {
         DistributedFileManagementImpl.REFERENCE.set(hazelcastInstance);
     }
@@ -123,6 +123,10 @@ public class DistributedFileManagementImpl implements DistributedFileManagement
         }
     }
 
+    private String encodeId(String rawId) throws OXException {
+        return services.getServiceSafe(DistributedFileUtils.class).encodeId(rawId);
+    }
+
     /**
      * Handles special {@link HazelcastInstanceNotActiveException} exception instance.
      *
@@ -166,7 +170,7 @@ public class DistributedFileManagementImpl implements DistributedFileManagement
             String url = map().get(id);
             if (url != null) {
                 try {
-                    return loadFile("http://" + url + "/" + id);
+                    return loadFile("http://" + url + "/" + encodeId(id));
                 } catch (IOException e) {
                     throw ManagedFileExceptionErrorMessage.IO_ERROR.create(e, e.getMessage());
                 }
@@ -187,7 +191,7 @@ public class DistributedFileManagementImpl implements DistributedFileManagement
             String url = map().get(id);
             if (url != null) {
                 try {
-                    URL remoteUrl = new URL("http://" + url + "/" + id);
+                    URL remoteUrl = new URL("http://" + url + "/" + encodeId(id));
                     HttpURLConnection con = (HttpURLConnection) remoteUrl.openConnection();
                     con.setRequestMethod("POST");
                     con.setConnectTimeout(CONNECT_TIMEOUT);
@@ -252,7 +256,7 @@ public class DistributedFileManagementImpl implements DistributedFileManagement
             String url = map().get(id);
             if (url != null) {
                 try {
-                    URL remoteUrl = new URL("http://" + url + "/" + id);
+                    URL remoteUrl = new URL("http://" + url + "/" + encodeId(id));
                     HttpURLConnection con = (HttpURLConnection) remoteUrl.openConnection();
                     con.setRequestMethod("DELETE");
                     con.setConnectTimeout(CONNECT_TIMEOUT);
@@ -274,7 +278,7 @@ public class DistributedFileManagementImpl implements DistributedFileManagement
     }
 
     /**
-     * Gets the associyated Hazelcast map.
+     * Gets the associated Hazelcast map.
      *
      * @return The Hazelcast map
      * @throws OXException If Hazelcast map cannot be returned
@@ -296,28 +300,30 @@ public class DistributedFileManagementImpl implements DistributedFileManagement
 
     private InputStream loadFile(String url) throws IOException {
         URL remoteUrl = new URL(url);
-        HttpURLConnection con = (HttpURLConnection) remoteUrl.openConnection();
-        con.setRequestMethod("GET");
-        con.setConnectTimeout(CONNECT_TIMEOUT);
-        con.setReadTimeout(READ_TIMEOUT);
-        con.connect();
 
-        boolean close = true;
+        HttpURLConnection con = null;
+        InputStream in = null;
         try {
+            con = (HttpURLConnection) remoteUrl.openConnection();
+            con.setRequestMethod("GET");
+            con.setConnectTimeout(CONNECT_TIMEOUT);
+            con.setReadTimeout(READ_TIMEOUT);
+            con.connect();
+
             int responseCode = con.getResponseCode();
-            if (responseCode == HttpURLConnection.HTTP_OK) {
-                final InputStream in = con.getInputStream();
-                close = false;
-                return in;
+            if (responseCode != HttpURLConnection.HTTP_OK) {
+                return null;
             }
-            return null;
+
+            in = con.getInputStream();
+            InputStream retval = in;
+            in = null; // Avoid premature closing
+            return retval;
         } finally {
-            if (close) {
-                try {
-                    // The only way to close a HttpURLConnection
-                    con.getInputStream().close();
-                } catch (Exception e) {
-                    // Ignore
+            if (in != null) {
+                Streams.close(in);
+                if (con != null) {
+                    con.disconnect();
                 }
             }
         }
