@@ -8,9 +8,11 @@ import org.osgi.framework.ServiceReference;
 import com.hazelcast.config.Config;
 import com.hazelcast.config.MapConfig;
 import com.hazelcast.core.HazelcastInstance;
+import com.hazelcast.instance.EndpointQualifier;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.dispatcher.DispatcherPrefixService;
 import com.openexchange.filemanagement.DistributedFileManagement;
+import com.openexchange.filemanagement.DistributedFileUtils;
 import com.openexchange.filemanagement.distributed.DistributedFileManagementImpl;
 import com.openexchange.hazelcast.configuration.HazelcastConfigurationService;
 import com.openexchange.osgi.HousekeepingActivator;
@@ -27,7 +29,7 @@ public class DistributedFileManagementActivator extends HousekeepingActivator {
 
     @Override
     protected Class<?>[] getNeededServices() {
-        return new Class<?>[] { HazelcastConfigurationService.class, ConfigurationService.class, DispatcherPrefixService.class, ThreadPoolService.class };
+        return new Class<?>[] { HazelcastConfigurationService.class, ConfigurationService.class, ThreadPoolService.class, DistributedFileUtils.class };
     }
 
     @Override
@@ -36,7 +38,6 @@ public class DistributedFileManagementActivator extends HousekeepingActivator {
         logger.info("Starting bundle: com.openexchange.filemanagement.distributed");
 
         HazelcastConfigurationService hazelcastConfig = getService(HazelcastConfigurationService.class);
-        final String prefix = getService(DispatcherPrefixService.class).getPrefix();
         final int port = getService(ConfigurationService.class).getIntProperty("com.openexchange.connector.networkListenerPort", 8009);
         final ServiceLookup services = this;
         final BundleContext context = this.context;
@@ -50,9 +51,9 @@ public class DistributedFileManagementActivator extends HousekeepingActivator {
                     DistributedFileManagementImpl.setHazelcastInstance(service);
 
                     // Address and map name
-                    String address = service.getCluster().getLocalMember().getSocketAddress().getHostName();
+                    String address = service.getCluster().getLocalMember().getSocketAddress(EndpointQualifier.MEMBER).getHostName();
                     String mapName = discoverMapName(service.getConfig(), logger);
-                    address = address + ":" + port + prefix;
+                    address = address + ":" + port;
 
                     // Clean-up task
                     Runnable shutDownTask = new Runnable() {
@@ -82,7 +83,7 @@ public class DistributedFileManagementActivator extends HousekeepingActivator {
                     DistributedFileManagementImpl.setHazelcastInstance(null);
 
                     DistributedFileManagementImpl distributedFileManagement = this.distributedFileManagement;
-                    if(distributedFileManagement != null) {
+                    if (distributedFileManagement != null) {
                         this.distributedFileManagement = null;
                         unregisterService(distributedFileManagement);
                         distributedFileManagement.cleanUp();
